
package MPlayer;
use strict;
use IPC::Open3 'open3';
use threads;
use threads::shared;

my $tv_thread_run : shared = 1;
my @moutput : shared = 1;

BEGIN {
	use vars qw ($VERSION @error
		     $mout $min $merror $mpid $tv_thread_run @moutput);
	$VERSION     = 0.04; # modified by cyclops
}

sub new
{
	my ($class, $filename, @parameters) = @_;
        push @error,  "Constructor needs \$filename\n" if (!defined ($filename));

	my $self = bless ({}, ref ($class) || $class);
	
        # small modification by cyclops
        # we don't check tv:// http:// filenames
	if (!($filename =~ /:\/\//) && !(-e $filename)) {
	    push @error, "File $filename does not exist";
	}
        
        $tv_thread_run = -1;

        eval {
            $mpid = open3($min, $mout, $merror, "mplayer", "-quiet", "-slave", @parameters, $filename);
        };
        if ($@) { #/^open3:/
            push @error, "Couldn't open MPlayer: $@";
            $mpid = undef;
        }
	elsif (!$mpid) {
	    push @error, "Couldn't open MPlayer: $!";
            $mpid = undef;
	}
        $tv_thread_run = 0;
        @moutput = ();
	
	return ($self);
}

#seek <value> [type=<0/\:1/\:2>]
#	Seek to some place in the movie.
#	Type 0 is a relative seek of +/\:- <value> seconds.
##	Type 1 seek to <value> % in the movie.
#	Type 2 is a seek to an absolute position of <value> seconds.
sub seek
{
    my $self = shift;
    my ($value, $type) = shift;
    return $self->argerr() if not defined $value or not defined $type;
    return undef unless $self->check_mplayer();
    
    $type = 0 unless defined $type;

    syswrite $min, "seek $value $type\n";
    return 1;
}

#	????
sub edl_mark
{
    my $self = shift;
    return -1;
}

#audio_delay <value>
#	Adjust the audio delay of value seconds
sub audio_delay 
{
    my $self = shift;
    my $value = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "audio_delay $value\n";
    return 1;
}

#quit
#	Quit MPlayer
sub quit
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "quit\n";

    return 1;
}

#pause
#	Pause/\:unpause the playback
sub pause
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "pause\n";
    return 1;
}

#grap_frames
#	Somebody know ?
sub grap_frames 
{
    my $self = shift;
    return $self->not_implemented();
}

#pt_step <value> [force=<value>]
#	Go to next/\:previous entry in the playtree.
sub pt_step 
{
    my $self = shift;
    my ($value, $force) = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();
    $force = "" unless defined $force;
    
    syswrite $min, "pt_step $value $force\n";
    return 1;
}

#pt_up_step <value> [force=<value>]
#	Like pt_step but it jumps to next/\:previous in the parent list.
sub pt_up_step
{
    my $self = shift;
    my ($value, $force) = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();
    $force = "" unless defined $force;

    syswrite $min, "pt_up_step $value $force\n";
    return 1;
}

#alt_src_step <value>
#	When more than one source is available it selects the next/\:previous one
#	(only supported by asx playlist).
sub alt_src_step
{
    my $self = shift;
    my $value = shift;
    return unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "alt_src_step $value\n";
    return 1;
}

#sub_delay <value> [abs=<value>]
#	Adjust the subtitles delay of +/\:- <value> seconds or set it to <value>
#	seconds when abs is non zero.
sub sub_delay 
{
    my $self = shift;
    my $value = shift;
    my $abs = shift;
    return $self->argerr() unless defined $value;
    $abs = 0 unless defined $abs;
    return undef unless $self->check_mplayer();

    syswrite $min, "sub_delay $value $abs\n";
    return 1;
}

#volume <dir>
#	Increase/\:decrease volume
sub volume
{
    my $self = shift;
    my $dir = shift;
    return $self->argerr() unless defined $dir;
    return undef unless $self->check_mplayer();

    syswrite $min, "volume $dir\n";
    return 1;
}

#use_master
#	Switch volume control between master and pcm.
sub use_master
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "use_master\n";
    return 1;
}

#mute
#	Mute/\:unmute sound output.
sub mute 
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "mute\n";
    return 1;
}

#[contrast|brightness|hue|saturation] <-100-100> [abs=<value>]
#	Set/\:Adjust video parameters.
sub contrast 
{
    my $self = shift;
    my $value = shift;
    my $abs = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();
    $abs = 0 unless defined $abs;

    syswrite $min, "contrast $value $abs\n";
    return 1;
}

sub brightness 
{
    my $self = shift;
    my $value = shift;
    my $abs = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();
    $abs = 0 unless defined $abs;

    syswrite $min, "brightness $value $abs\n";
    return 1;
}

sub hue
{
    my $self = shift;
    my $value = shift;
    my $abs = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();
    $abs = 0 unless defined $abs;

    syswrite $min, "hue $value $abs\n";
    return 1;
}

sub saturation
{
    my $self = shift;
    my $value = shift;
    my $abs = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();
    $abs = 0 unless defined $abs;

    syswrite $min, "saturation $value $abs\n";
    return 1;
}

#frame_drop [type=<value>]
#	Toggle/\:Set frame dropping mode.
sub frame_drop 
{
    my $self = shift;
    my $value = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "frame_drop\n";
    return 1;
}

#sub_alignment <value>
#	Set subtitle alignment.
sub sub_alignment 
{
    my $self = shift;
    my $value = shift;
    return unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "sub_alignment $value\n";
    return 1;
}

#sub_visibility
#	Toggle subtitle visibility.
sub sub_visibility
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "sub_visibility\n";
    return 1;
}

#sub_pos <value>
#	Adjust subtitles position.
sub sub_pos 
{
    my $self = shift;
    my $value = shift;
    return unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "sub_pos $value\n";
    return 1;
}

#vobsub_lang
#	Switch the subtitle language when using VOBSub subtitles.
sub vobsub_lang
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "vobsub_lang\n";
    return 1;
}

#get_percent_pos
#	???
sub get_percent_pos
{
    my $self = shift;
    return $self->not_implemented();

}

#get_time_length
#	???
sub get_time_length
{
    my $self = shift;
    return $self->not_implemented();

}

#vo_fullscreen
#	Switch to fullscreen mode.
sub vo_fullscreen
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "vo_fullscreen\n";
    return 1;
}

#panscan value [value2]
#	???
sub panscan 
{
    my $self = shift;
    my $value = shift;
    return $self->not_implemented();
}

#loadfile url
#	Load the given file/\:url.
sub loadfile 
{
    my $self = shift;
    my $url = shift;
    return $self->argerr() unless defined $url;
    return undef unless $self->check_mplayer();

    syswrite $min, "loadfile $url\n";
    return 1;
}

#loadlist file
#	Load the given playlist file.
sub loadlist 
{
    my $self = shift;
    my $file = shift;
    return $self->argerr() unless defined $file;
    return undef unless $self->check_mplayer();
    
    syswrite $min, "loadlist $file\n";
    return 1;
}

#change_rectangle val1 val2
#	??? used by the rectangle video filter.
sub change_rectangle
{
    my $self = shift;
    my ($val1, $val2) = shift;
    return $self->not_implemented();
}

#dvdnav button
#	Press the given dvdnav button.
#	1 is up, 2 is down, 3 is left, 4 is right, 5 is menu and 6 is select.
sub dvdnav
{
    my $self = shift;
    my $button = shift;
    return $self->argerr() unless defined $button;
    return undef unless $self->check_mplayer();

    syswrite $min, "dvdnav $button\n";
    return 1;
}

#gui_[loadsubtitle|about|play|stop]
#	GUI actions
sub gui_loadsubtitle
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "gui_loadsubtitle\n";
    return 1;
}

sub gui_about
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "gui_about\n";
    return 1;
}

sub gui_play
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "gui_play\n";
    return 1;
}

sub gui_stop
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "gui_stop\n";
    return 1;
}

# added by cyclops, 2004-05-24
# internal function, do not use!
sub tv_thread
{
    my ($out_mode, $function, $param) = @_;
    my $line = "";
    my $ret = -1;
    my $c;
    if ($tv_thread_run == -1) {
        push @error, "MPlayer not running, can't start thread\n";
        return;
    }
    $tv_thread_run = 1;
    if ($out_mode == 1) {
        open FH, ">/tmp/mplayertv.log";
        select FH;
    }
    if ($out_mode == 2) {
        select STDOUT; $| = 1;      # make unbuffered
    }
    while ($ret != 0) {
        $ret = sysread $mout, $c, 1;
        next unless $ret;
        print $c if ($out_mode >= 1);
        if ($c eq "\n") {
            if ($line =~ /^Selected channel.* - ([\s\w]+) \(.*/) {
                my $new_channel = $1;
                $new_channel =~ s/ /_/g;
                &$function ($param, $new_channel);
            }
            else {
                push @moutput, $line;
            }
            $line = "";
        }
        else {
            $line .= $c;
        }
    }
    select STDOUT;
    close FH if ($out_mode == 1);
    $tv_thread_run = 0;
}

# added by cyclops, 2004-05-24
# returns true if thread is running (mplayer running)
sub tv_thread_running
{
    my $self = shift;
    return undef unless $self->check_mplayer();
    return $tv_thread_run;
}

# added by cyclops, 2004-05-24
# if user steps channel run callback function
sub tv_set_channel_step_callback
{
    my $self = shift;
    my $out_mode = shift;
    my $function = shift;
    my $param = shift;
    return undef unless $self->check_mplayer();
    if ($tv_thread_run) {
        push @error, "Thread already running";
        return undef;
    }
    my $thread  = threads->create(\&tv_thread, $out_mode, $function, $param);
    return 1;
}

#tv_step_channel <dir>
#	Select next/\:previous tv channel.
sub tv_step_channel
{
    my $self = shift;
    my $dir = shift;
    return $self->argerr() unless defined $dir;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_step_channel $dir\n";
    return 1;
}

#tv_step_norm
#	Change TV norm.
sub tv_step_norm
{
    my $self = shift;
    return undef unless $self->check_mplayer();
   
    syswrite $min, "tv_step_norm\n";
    return 1;
}

#tv_step_chanlist
#	Change channel list.
sub tv_step_chanlist
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_step_chanlist\n";
    return 1;
}

#tv_set_channel channel
#	Set the current TV channel.
sub tv_set_channel 
{
    my $self = shift;
    my $channel = shift;
    return unless defined $channel;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_channel $channel\n";
    return 1;
}

#tv_last_channel
#	Set the current TV channel to the last one.
sub tv_last_channel
{
    my $self = shift;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_last_channel\n";
    return 1;
}

#tv_set_freq <frequency in MHz>
#	Set the tv tuner frequency.
sub tv_set_freq
{
    my $self = shift;
    my $freq = shift;
    return $self->argerr() unless defined $freq;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_freq $freq\n";
    return 1;
}

#tv_set_norm <norm>
#	Set the tv tuner norm. PAL, SECAM, NTSC and so on..
sub tv_set_norm
{
    my $self = shift;
    my $norm = shift;
    return $self->argerr() unless defined $norm;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_norm $norm\n";
    return 1;
}

#tv_set_brightness [-100:100]
#	Set tv tuner brightness.
sub tv_set_brightness 
{
    my $self = shift;
    my $value = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_brightness $value\n";
    return 1;
}

#tv_set_contrast [-100:100]
#	Set tv tuner contrast.
sub tv_set_contrast
{
    my $self = shift;
    my $value = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_contrast $value\n";
    return 1;
}

#tv_set_hue [-100:100]
#	Set tv tuner hue.
sub tv_set_hue
{
    my $self = shift;
    my $value = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_hue $value\n";
    return 1;
}

#tv_set_saturation [-100:100]
#	Set tv tuner saturation.
sub tv_set_saturation 
{
    my $self = shift;
    my $value = shift;
    return $self->argerr() unless defined $value;
    return undef unless $self->check_mplayer();

    syswrite $min, "tv_set_saturation $value\n";
    return 1;
}

sub error 
{
    my $self = shift;
    if (wantarray) {
	return @error;
    } else {
	return pop @error;
    }
}

sub output
{
    my $self = shift;
    if (wantarray) {
	return @moutput;
    } else {
	return join ("\n", @moutput);
    }
}

# TODO fix bug!
# bug: if user quitting with q then check_mplayer returns 1!
sub check_mplayer
{
    my $self = shift;
    return 0 unless $mpid;
    if (kill 0, $mpid) {
       return 1;
    } else {
        push @error, "MPlayer died for some reason";
        return 0;
    }
}

sub argerr
{
    my $self = shift;
    my $call = caller();

    push @error, "Function $call was called with the wrong arguments";
    return undef;
}

sub not_implemented
{
    my $self = shift;
    my $call = caller();

    push @error, "Function $call is not implemented";
    return undef;
}


1; #this line is important and will help the module return a true value
__END__


########################################### main pod documentation begin ##
# Below is the stub of documentation for your module. You better edit it!


=head1 NAME

MPlayer - An OO interface to MPlayer for Perl (modified by Peter Ivanov)

=head1 SYNOPSIS

  use MPlayer
  
  my $mp = new MPlayer("./opensource.ogg", "-ao" => "oss");

  sleep 10;

  $mp->mute();

  $mp->quit();

  $mp = new MPlayer("some_movie.avi", "-cache" => 1024);

  $mp->vo_fullscreen();

  $mp->pause();

  my @error = $mp->error;
  print join "\n", @error;

  #Show last error:
  my $error = $mp->error
  print $error;

  $mp->tv_step_channel(1);
  $mp->tv_set_channel(2);

=head1 DESCRIPTION

MPlayer.pm is a simple interface to MPlayers slave mode command and
can be used inside a perlscript.

=head1 NOTE

MPlayer.pm does not keep block, i.e. if your script exits before the
sound file is finished, it will just kill MPlayer. This module
might be used to write interfaces in Perl::Tk or Curses::UI which
have a Mainloop for flowcontrol.

=head1 GENERAL

MPlayer implements all calls documented in DOCS/tech/slave.txt. 
Calls will return a true value on success and 'undef' otherwise.
Some calls are not documented at all, they return 'undef' defaultly.
If an error occurs you can access those errors through a call
to $mp->error() which will give you a string error description.

See DOCS/tech/slave.txt in the mplayer source tree for all calls
and their options.

The tv_ calls can be called thourgh $mp->tv_...

=head1 USAGE

=over 4

=item new( filename, mplayer_opts... )

Filename is mandatory, MPlayer needs a given filename to start
up. Mplayer_opts can be any of MPlayers command line options as
an anonymous hash (e.g. "-cache" => 1024 ).

=item error() 

Returns descriptions of errors that have occured in MPlayer.pm.
In array context it returns all errors, in scalar context the
last error that occured.


=back

=head1 INSTALL

At the very least you should be able to use this set of instructions
to install the module...

    perl Makefile.PL
    make
    make test
    make install


If you are on a windows box you should use 'nmake' rather than 'make'.


=head1 BUGS

This is the first version, mostly untested when it comes to TV stuff.
If you find a bug please report it to the author.


=head1 AUTHOR

	Marcus Thiesen
	marcus@cpan.org
	http://www.thiesenweb.de

        Peter Ivanov
        http://www.mplayertv.tk

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).
mplayer

=cut
